@@ -1,15 +1,14 @@
 #' Clear the lintr cache
 #'
-#' @param file filename whose cache to clear.  If you pass \code{NULL}, it will
-#' delete all of the caches.
-#' @param path directory to store caches.  Reads option 'lintr.cache_directory'
-#' as the default.
+#' @param file filename whose cache to clear. If you pass `NULL`, it will delete all of the caches.
+#' @param path directory to store caches. Reads option 'lintr.cache_directory' as the default.
 #' @return 0 for success, 1 for failure, invisibly.
 #' @export
 clear_cache <- function(file = NULL, path = NULL) {
-  read_settings(file)
-
   if (is.null(path)) {
+    # Only retrieve settings if `path` isn't specified.
+    # Otherwise, other settings may inadvertently be loaded, such as exclusions.
+    read_settings(file)
     path <- settings$cache_directory
   }
 
@@ -20,6 +19,20 @@ clear_cache <- function(file = NULL, path = NULL) {
   unlink(path, recursive = TRUE)
 }
 
+define_cache_path <- function(cache) {
+  if (isTRUE(cache)) {
+    settings$cache_directory
+  } else if (is.character(cache)) {
+    cache
+  } else {
+    character()
+  }
+}
+
+define_cache_key <- function(filename, inline_data, lines) {
+  if (inline_data) list(content = get_content(lines), TRUE) else filename
+}
+
 
 get_cache_file_path <- function(file, path) {
   # this assumes that a normalized absolute file path was given
@@ -27,37 +40,45 @@ get_cache_file_path <- function(file, path) {
 }
 
 load_cache <- function(file, path = NULL) {
-  read_settings(file)
-
-  if (is.null(path)) {
-    path <- settings$cache_directory
+  if (is.null(path) || length(path) == 0L) {
+    return()
   }
-
   env <- new.env(parent = emptyenv())
 
   file <- get_cache_file_path(file, path)
   if (file.exists(file)) {
-    load(file = file, envir = env)
+    tryCatch(
+      load(file = file, envir = env),
+      warning = function(w) {
+        invokeRestart("muffleWarning")
+      },
+      error = function(e) {
+        warning(
+          "Could not load cache file '", file, "':\n",
+          conditionMessage(e)
+        )
+      }
+    )
   } # else nothing to do for source file that has no cache
 
   env
 }
 
 save_cache <- function(cache, file, path = NULL) {
-  read_settings(file)
-
-  if (is.null(path)) {
-    path <- settings$cache_directory
+  if (is.null(cache)) {
+    return()
   }
-
   if (!file.exists(path)) {
-    dir.create(path)
+    dir.create(path, recursive = TRUE)
   }
 
   save(file = get_cache_file_path(file, path), envir = cache, list = ls(envir = cache))
 }
 
 cache_file <- function(cache, filename, linters, lints) {
+  if (is.null(cache)) {
+    return()
+  }
   assign(
     envir = cache,
     x = digest_content(linters, filename),
@@ -67,6 +88,9 @@ cache_file <- function(cache, filename, linters, lints) {
 }
 
 retrieve_file <- function(cache, filename, linters) {
+  if (is.null(cache)) {
+    return(NULL)
+  }
   mget(
     envir = cache,
     x = digest_content(linters, filename),
@@ -77,11 +101,15 @@ retrieve_file <- function(cache, filename, linters) {
 }
 
 cache_lint <- function(cache, expr, linter, lints) {
+  if (is.null(cache)) {
+    return()
+  }
   assign(
     envir = cache,
     x = digest_content(linter, expr),
     value = lints,
-    inherits = FALSE)
+    inherits = FALSE
+  )
 }
 
 retrieve_lint <- function(cache, expr, linter, lines) {
@@ -91,15 +119,26 @@ retrieve_lint <- function(cache, expr, linter, lines) {
     mode = "list",
     inherits = FALSE
   )
-  lints[] <- lapply(lints, function(lint) {
-    lint$line_number <- find_new_line(lint$line_number, unname(lint$line), lines)
-    lint
-  })
+  for (i in seq_along(lints)) {
+    new_line_number <- find_new_line(
+      lints[[i]]$line_number,
+      unname(lints[[i]]$line),
+      lines
+    )
+    if (is.na(new_line_number)) {
+      return(NULL)
+    } else {
+      lints[[i]]$line_number <- new_line_number
+    }
+  }
   cache_lint(cache, expr, linter, lints)
   lints
 }
 
 has_lint <- function(cache, expr, linter) {
+  if (is.null(cache)) {
+    return(FALSE)
+  }
   exists(
     envir = cache,
     x = digest_content(linter, expr),
